#!/usr/bin/perl

# Copyright (C) 2006-2007 by Pawel Pilarczyk.
#
# This file is part of my academic software package.  It is free software;
# you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this software; see the file "license.txt".  If not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
# MA 02111-1307, USA.

# Started on January 17, 2006. Last revision: December 5, 2007.

die <<"END" if (scalar (@ARGV) < 2);
DAT2CELL, ver. 0.01. Copyright (C) 2006-2007 by Pawel Pilarczyk.
This is free software. No warranty. Consult 'license.txt' for details.

This script extracts cells from a multiwork subdivision scheme log file,
see 'chomp/multiwork/mwsubdiv' in the Computational Homology Package.
It produces a list of cells separately for each result value and also runs
the program 'cell2bmp' (included in the CHomP software package,
see http://chomp.rutgers.edu/) to create the corresponding illustration.

Call this script with the input file name and a base for the series
of output files: file.dat out

The script will create a series of files named outN.cel, where the N
will be replaced with the result value for the corresponding cells.
Based on the contents of these files, this script will also produce
two files named out_c.bmp and out_g.bmp with a color and grey-scale
illustration of these sets of cells, respectively.
END

my $inname = $ARGV[0];
my $out = $ARGV[1];
my %cells;
my %bitmaps;

# Scans the file to determine the highest subdivision level
# at which full boxes appear in the file.
sub scanlevel
{
	my $filename = shift;
	open (F, $filename) or die "Can't open '$filename'.\n";
	my $maxlevel = 0;
	while (<F>)
	{
		next unless m/^\* \d+ \d+ (\d+) \([\d-, ]+\)/;
		my $level = $1;
		$maxlevel = $level if ($maxlevel < $level);
	}
	close F;
	return $maxlevel;
}

my $maxlevel = scanlevel ($inname);
print "Highest subdivision level found: $maxlevel.\n";

# Transforms a cube at the given level to a cell of appropriate size.
sub cub2cell
{
	my $level = shift;
	my $cube = shift;
	$cube =~ s/\((.*)\)/\1/;
	my @coord = split (/[, ]/, $cube);
	my $size = 2 ** ($maxlevel - $level);
	my @left = map {$_ * $size} @coord;
	my @right = map {$_ + $size} @left;
	return "[(" . join (',', @left) . ")(" . join (',', @right) . ")]";
}

open (FILE, $inname) or die "Can't open '$inname'.\n";
while (<FILE>)
{
	next unless m/^\* (\d+) (\d+) (\d+) (\([\d-, ]+\))/;
	my $number = $1;
	my $result = $2;
	my $level = $3;
	my $cube = $4;
	my $cubfilename = "${out}${result}.cel";
	$cells{$cubfilename} .= cub2cell ($level, $cube) . "\n";
	my $bmpfilename = "${out}_c";
	$bitmaps{$bmpfilename}[$result] = $cubfilename;
	$bmpfilename = "--grey ${out}_g";
	$bitmaps{$bmpfilename}[$result] = $cubfilename;
}
close FILE;

foreach my $name (keys %cells)
{
	open OUTFILE, ">$name";
	print OUTFILE $cells{$name};
	close OUTFILE;
}

foreach my $name (keys %bitmaps)
{
	my $args = join (' ', @{ $bitmaps{$name} });
	$args =~ s/^.*? //;
	$args =~ s/ +/ /g;
	$args =~ s/^ //;
	print `cell2bmp -s1 -p1 $args ${name}.bmp`;
}

